

/*************************** FdShm.c *****************************/
/*                                                               */
/*               FRAMES IN SHARED MEMORY                         */
/*                     LIBRARY                                   */
/* MODULE  : FdShm.c                                             */
/* AUTHOR  : D. Verkindt, LAPP (verkindt@lapp.in2p3.fr)          */
/* VERSION : v5r0p3                                              */
/* DATE: 27 Jan 2006                                             */
/*                                                               */
/*   This library allows a server (for instance the Main Frame   */
/*   Builder producer) to create a shared memory segment and     */
/*   to put frames into this shared memory. It allows also       */
/*   several consumers (for instance the Main Frame Builder      */
/*   consumers) to read those frames in shared memory.           */
/*                                                               */
/************************** FdShm.c ******************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <unistd.h>
#include "dataDisplay.h"

#ifdef MEMWATCH
#include "memwatch.h"
#endif


/*------------------------------------------ FdShmInfoFree --------*/

void FdShmInfoFree (struct ShmInfo *shm)
{
 if (shm == NULL) return;
 if (shm->frameSh)    free(shm->frameSh);
 if (shm->frameSize)  free(shm->frameSize);
 free(shm);
}


/*----------------------------------------- FdShmGetIwrite ----------*/

long FdShmGetIwrite (struct ShmInfo *shm)
{
 return(shm->iwrite);
}


/*----------------------------------------- FdShmGetnFrameMax ----------*/

long FdShmGetnFrameMax (struct ShmInfo *shm)
{
 long nFrameMax;
 if (shm->dynamic > 0) nFrameMax = shm->iwriteMax;
 else                  nFrameMax = shm->nFrameMax;
 return(nFrameMax);
}


/*----------------------------------------- FdShmSetDynamic ----------*/

void FdShmSetDynamic (struct ShmInfo *shm, long dynamic)
{
 shm->dynamic = dynamic;
 return;
}


/*------------------------------------------ FdShmFree --------*/

void FdShmFree (struct ShmInfo *shm, char *name)
{
 key_t key;
 int shmid;
 
 if (name == NULL  ||  shm == NULL) return;

 key = FdShmMakeKey(name);

 if ((shmid = shmget(key, 0, 0)) != -1) 
    {shmctl(shm->shmid, IPC_RMID, 0);
     if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0) printf("FdShmFree> %s Shared memory segment marked for deletion\n", name);
    }
 else
    {if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0) printf("FdShmFree> Consumer %s detached\n", name);
     shm->nConsumer--;
    }

 shmdt((void *)shm);

 return;
}


/*------------------------------------------ FdShmClose --------*/
/* only for consumers */

void FdShmClose (struct ShmInfo *shm, char *name)
{
 if (name == NULL  ||  shm == NULL) return;

 if (strcmp(shm->producerName, name) == 0) 
    {printf("FdShmClose> has no effect for a producer (%s). Use FdShmFree.\n", name);
     return;
    }
 else
    {if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0) printf("FdShmClose> consumer %s detached from shared memory\n", name);
     shm->nConsumer--;
    }

 shmdt((void *)shm);

 return;
}


/*-------------------------------------------  FdShmOpen -------*/
/*---------- this function is only for a consumer ----*/

struct ShmInfo *FdShmOpen (char *idName, char *name)
{
 struct ShmInfo *temp;
 void *segptr;
 key_t key;
 int shmid;

 shmid = atoi(idName);

 if (shmid == 0)
    { 
     key = FdShmMakeKey(idName);
     if ((shmid = shmget(key, 0, 0)) == -1)  return(NULL);
     if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0) printf("FdShmOpen> Connected to shm segment: key=%d  shmid=%d\n", key, shmid);
    }


 /*--- Attach (map) the shared memory segment into the current process ---*/

 if ((segptr = shmat(shmid, 0, 0)) == (void *)-1)
    {printf("FdShmOpen> Could not attach Shm segment shmid=%d\n", shmid);
     return(NULL);
    }

 if (segptr == NULL)
    {printf("FdShmOpen> shm segptr is NULL\n");
     return(NULL);
    }

 temp = (struct ShmInfo *)segptr;

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0)
    {printf("FdShmOpen> segptr=%lx(%ld)  shm=%lx(%ld) (shmid=%d)\n",
         (long)temp->segptr, (long)temp->segptr, (long)temp, (long)temp, shmid);
    }

 temp->nConsumer++;

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0) 
    {printf("FdShmOpen> New consumer %s (%ld) connected to shm %s\n", name, temp->nConsumer, temp->producerName);}

 return(temp);
}

/*------------------------------------------- FdShmMakeKey -------*/

int FdShmMakeKey (char *name)
{
 int i, length, key, val, nbytes;
 
 length = strlen(name);

 key = 0;
 for (i=0; i<length; i=i+4)
     {val = 0;
      if (length-i < 4) nbytes=length-i;
      else              nbytes = 4;
      memcpy(&val, &name[i], nbytes);
      key += val;
      if (key < 0) key += INT_MAX;
     }

 return(key);
}


/*-------------------------------------------  FdShmCsmIni -------*/

long FdShmCsmIni (struct ShmInfo *temp, long *shmiread)
{
 if (temp) *shmiread = temp->iwrite - 1;

 return(*shmiread);
}


/*------------------------------------------- FdShmRead -------------*/

struct FrameH *FdShmRead (struct ShmInfo *shm, long *shmiread, long compress, int *nBytes)
{
 struct FrameH *frame;
 long address, iread, iwrite;

 if (shm == NULL) {*nBytes = -1; return((struct FrameH*)NULL);}

 if (shm->dynamic > 0)
    {frame = FdShmReadDynamic (shm, shmiread, compress, nBytes);
     return(frame);
    }

 iwrite = shm->iwrite;

 /*--- don't try to read a frame currently written ---*/
 if (*shmiread >= iwrite)  {*nBytes = -2; return((struct FrameH*)NULL);}

 /*--- if too late with respect to producer, go behind it (skipping nFrameMax frames) ---*/
 if (*shmiread <= iwrite - shm->nFrameMax)
    {
     printf("FdShmRead> late in reading shm (%d/%d). Skipping %d frames, I go to iread=%d\n",
             *shmiread, iwrite, iwrite-*shmiread, iwrite-1);
     *shmiread = iwrite - 1;
    }

 /*--- ok, read the frame in share memory ---*/

 iread = (*shmiread) % shm->nFrameMax;

 *nBytes = (int)(shm->frameSize[iread]);
 address = shm->frameSh[iread] + (long) shm - (long) shm->segptr;

 if (*nBytes <= 0)
    {
     *nBytes = -3;
     printf("FdShmRead> nBytes <=0\n");
     return((struct FrameH*)NULL);
    }
 if (address == (long)NULL)
    {*nBytes = -4;
     printf("FdShmRead> address=NULL\n");
     return((struct FrameH*)NULL);
    }

 frame = FrameReadFromBuf((char *)address, *nBytes, compress);

 if (frame == NULL)
    {printf("FdShmRead> Not been able to read frame (%d bytes) at adress %ld+%ld (%ld)  (iread=%ld iwrite=%ld) (compress=%ld)\n",
            *nBytes, (long)shm->segptr, shm->frameSh[iread], address, *shmiread, iwrite, compress);
     *nBytes = 0;
    }

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>2)
    {
     if (frame)
        {printf("FdShmRead: frameGPS=%d  frameSize=%d  iwrite=%ld  iread=%ld  time=%ld\n",
                        frame->GTimeS,  *nBytes, shm->iwrite, *shmiread, (long)time(NULL));
        }
     else
        {printf("FdShmRead: frame=NULL frameSize=%d  iwrite=%ld  iread=%ld  time=%ld\n",
                        *nBytes, shm->iwrite, *shmiread, (long)time(NULL));
        }
    }

 (*shmiread)++;

 return(frame);
}


/*------------------------------------------- FdShmReadDynamic  -------------*/

struct FrameH *FdShmReadDynamic (struct ShmInfo *shm, long *shmiread, long compress, int *nBytes)
{
 struct FrameH *frame;
 long address, iwrite, iwriteMod, iwriteMax;
 long writeAddress, readAddress, writeSize;
 static long ireadMod=-1;
 long segEnd;

 if (shm == NULL) {*nBytes = -1; return((struct FrameH *)NULL);}

 iwrite    = shm->iwrite;
 iwriteMod = shm->iwriteMod;
 iwriteMax = shm->iwriteMax;

 if (*shmiread >= iwrite) {*nBytes = -2; return((struct FrameH *)NULL);}

 segEnd  = shm->frameSh[0] + shm->size - shm->statusSize;

 if (ireadMod == -1)
    {if (iwriteMod > 0) ireadMod = iwriteMod - 1;
     else               ireadMod = iwriteMax;
     *shmiread = iwrite - 1;
    }

 if (shm->frameSh[ireadMod] == 0) sleep(1);

 if (shm->frameSh[ireadMod] > segEnd-shm->frameSizeMax || shm->frameSh[ireadMod] == 0) ireadMod = 0;

 writeAddress = shm->frameSh[iwriteMod];
 writeSize    = shm->frameSize[iwriteMod];
 readAddress  = shm->frameSh[ireadMod];

 if ((writeAddress+writeSize >= readAddress) && (readAddress > writeAddress))
    {
     printf("FdShmRead> late in reading shm (iread=%ld iwrite=%ld). Skeeping %ld frames, I go to iread=%d "
            " (readAddress[%ld]-writeAddress[%ld]=%lx-%lx=%ld  writeSize=%ld)\n",
            *shmiread, iwrite, iwrite-*shmiread, iwrite-1, ireadMod, iwriteMod,
            readAddress, writeAddress, readAddress-writeAddress, writeSize);
     *shmiread = iwrite - 1;
     if (iwriteMod > 0) ireadMod = iwriteMod - 1;
     else               ireadMod = iwriteMax;
    }

 *nBytes = shm->frameSize[ireadMod];
 address = shm->frameSh[ireadMod] + (long) shm - (long) shm->segptr;

 if (*nBytes <= 0)
    {*nBytes = -3;
     printf("FdShmRead> nBytes <=0\n");
     return(NULL);
    }
 if (address == (long)NULL)
    {*nBytes = -4;
     printf("FdShmRead> address=NULL\n");
     return(NULL);
    }

 frame = FrameReadFromBuf((char *)address, *nBytes, compress);

 if (frame == NULL)
    {printf("FdShmRead> Not been able to read frame (%d bytes) at adress %ld+%ld (%ld)\n"
            "  (iread=%ld iwrite=%ld ireadMod=%ld iwriteMod=%ld iwriteMax=%ld) (compress=%ld)\n",
             *nBytes, (long)shm->segptr, shm->frameSh[ireadMod], address,
             *shmiread, iwrite, ireadMod, iwriteMod, iwriteMax, compress);
     *nBytes = 0;
    }

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>2)
    {
     if (frame)
        {printf("FdShmRead: frameGPS=%d  frameSize=%d  iwrite=%ld  iread=%ld  iwriteMod=%ld  ireadMod=%ld  iwriteMax=%ld  time=%ld\n",
                        frame->GTimeS,  *nBytes, iwrite, *shmiread, iwriteMod, ireadMod, iwriteMax, (long)time(NULL));
        }
     else
        {printf("FdShmRead: frame=NULL frameSize=%d  iwrite=%ld  iread=%ld  iwriteMod=%ld  ireadMod=%ld  iwriteMax=%ld  time=%ld\n",
                         *nBytes, iwrite, *shmiread, iwriteMod, ireadMod, iwriteMax, (long)time(NULL));
        }
    }

 (*shmiread)++;
 ireadMod++;

 return(frame);
}

